/* (C) 2024 YiRing, Inc. */
package com.yiring;

import cn.hutool.core.lang.Console;
import com.yiring.common.core.RetryExecutor;
import org.junit.jupiter.api.Test;
import org.springframework.boot.SpringBootConfiguration;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.retry.annotation.EnableRetry;

@SpringBootConfiguration
@SpringBootTest
@EnableRetry
public class RetryTests {

    RetryExecutor retry = new RetryExecutor();

    /**
     * 简易重试，超过默认次数会抛出异常
     */
    @Test
    void def() {
        Integer value = retry.run(ctx -> {
            Console.log("retry test, {}", ctx.getRetryCount());

            String text = "0.0";
            if (ctx.getRetryCount() > 1) {
                text = "1";
            }

            return Integer.parseInt(text);
        });
        Console.log("[default] retry test result: {}", value);
    }

    /**
     * 默认的退回策略
     */
    @Test
    void recover() {
        Integer value = retry.run(
            ctx -> {
                Console.log("retry test, {}", ctx.getRetryCount());
                throw new RuntimeException("test");
            },
            ctx -> {
                Console.log("retry recover, {}", ctx.getRetryCount());
                return 666;
            },
            template -> RetryExecutor.setSimplePolicy(template, 2, 1000L)
        );
        Console.log("[recover] retry test result: {}", value);
    }

    /**
     * 重试 2 次，每次间隔 5s，失败默认退回 null
     */
    @Test
    void custom() {
        Integer value = retry.run(
            ctx -> {
                Console.log("retry test, {}", ctx.getRetryCount());
                throw new RuntimeException("test");
            },
            ctx -> {
                Console.log("retry recover, {}", ctx.getRetryCount());
                return null;
            },
            template -> RetryExecutor.setSimplePolicy(template, 2, 5000L)
        );
        Console.log("[recover] retry test result: {}", value);
    }
}
